#ifndef JTHREAD_H__
#define JTHREAD_H__

#include <sys/time.h>
#include <jni.h>
#include <jvmpi.h>
#include <timerstack.h>
#include <hash.h>
#include <jmp.h>
#include <cls.h>
#include <method.h>

/** This is the structure used for a java thread. */ 
struct jthread {
    char* thread_name;       /** The name of the thread. */
    char* group_name;        /** The name of the thread group. */
    char* parent_name;       /** The name of the parent thread. */
    jobjectID thread_id;     /** The id of the thread. */
    JNIEnv* env_id;          /** The environment id of the thread. */
    jlong contendtime;        /** The time this thread has spend waiting for locks.. */
};

/** Create a new thread structure with the given attributes. */
jthread* jthread_new (const char* thread_name, const char* group_name, 
		      const char* parent_name, jobjectID thread_id,
		      JNIEnv* env_id);

/** Delete the given thread structure. */
void jthread_free (jthread* t);

/** The hash function for threads. */
size_t jthread_hash_func (void* c, size_t len);

/** Check if two threads are equal.
 *  return 0 if the threads are equal.
 */
int jthread_cmp_func (void* c1, void* c2);

/** Get the environment id of the given thread. */
JNIEnv* jthread_get_env_id (jthread* t);

/** Set the environment id of the given thread. */
void jthread_set_env_id (jthread* t, JNIEnv* env_id);

/** The given thread has entered a method at time tv. 
 */
void jthread_method_entry (timerstack* s, jmethodID method_id, const struct timeval* tv, jlong tval);

/** The given thread has exited from a method.
 */
void jthread_method_exit (timerstack* s, method* m, const struct timeval* tv, jlong tval);

/** The given thread has is waiting for a monitor. */
void jthread_contenation_enter (timerstack* s, obj* monitor, const struct timeval* tv, jthread* t, jlong tval);

/** The given thread got the monitor it was waiting for. */
void jthread_contenation_entered (timerstack* s, obj* monitor, const struct timeval* tv, jthread* t, jlong tval);

/** Get the name of the given thread. */
char* jthread_get_thread_name (jthread* t);

/** Get the name of the group for the given thread. */
char* jthread_get_group_name (jthread* t);

/** Get the name of the parent for the given thread. */
char* jthread_get_parent_name (jthread* t);


#endif   /* JTHREAD_H__ */
