#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <cls.h>
#include <hash.h>
#include <jmp.h>
#include <ui.h>
#include <method.h>
#include <cls.h>
#include <obj.h>
#include <jthread.h>
#include <mvector.h>
#include <methodtime.h>
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <jmp-config.h>

static char* Q = "?";
static char* SYS_INIT = "[system init]";
static char sarena[64];
static char is_array[64];
static char size[64];

enum {
    OD_CLASS,
    OD_ARENA,
    OD_IS_ARRAY,
    OD_SIZE,
    OD_ALLOC_CLASS,
    OD_ALLOC_METHOD,
    OD_OBJECT,
    ODN_COLUMNS
};

typedef struct {
    GtkWidget *window;
    GtkListStore *clist;
    gint       row;
    void      *data;
} wret;

typedef struct {
    GtkListStore *clist;
    gint       row;    
} col_click_info;

static gint sort_by_class_name (GtkTreeModel *model,
				GtkTreeIter *a,
				GtkTreeIter *b,
				gpointer user_data) {
    obj* o1;
    obj* o2;
    cls* cp1;
    cls* cp2;
    char* cn1;
    char* cn2;

    gtk_tree_model_get (model, a, OD_OBJECT, &o1, -1);
    gtk_tree_model_get (model, b, OD_OBJECT, &o2, -1); 
    cp1 = get_class (obj_get_class_id (o1));
    cp2 = get_class (obj_get_class_id (o2));
    cn1 = cp1 ? cp1->name : Q;
    cn2 = cp2 ? cp2->name : Q;
    return (strcmp (cn1, cn2));    
}


static gint sort_by_arena (GtkTreeModel *model,
			   GtkTreeIter *a,
			   GtkTreeIter *b,
			   gpointer user_data) {
    obj* o1;
    obj* o2;
    gtk_tree_model_get (model, a, OD_OBJECT, &o1, -1);
    gtk_tree_model_get (model, b, OD_OBJECT, &o2, -1); 
    return o1->arena_id - o2->arena_id;
}

static gint sort_by_is_array (GtkTreeModel *model,
			      GtkTreeIter *a,
			      GtkTreeIter *b,
			      gpointer user_data) {
    obj* o1;
    obj* o2;
    gtk_tree_model_get (model, a, OD_OBJECT, &o1, -1);
    gtk_tree_model_get (model, b, OD_OBJECT, &o2, -1); 
    return o1->is_array - o2->is_array;
}

static gint sort_by_size (GtkTreeModel *model,
			  GtkTreeIter *a,
			  GtkTreeIter *b,
			  gpointer user_data) {
    obj* o1;
    obj* o2;
    gtk_tree_model_get (model, a, OD_OBJECT, &o1, -1);
    gtk_tree_model_get (model, b, OD_OBJECT, &o2, -1); 
    return o1->size - o2->size;
}

static gint sort_by_alloc_class_and_method (GtkTreeModel *model,
					    GtkTreeIter *a,
					    GtkTreeIter *b,
					    gpointer user_data) {
    cls* c1 = NULL;
    cls* c2 = NULL;
    char* cn1;
    char* cn2;
    int i;
    obj* o1;
    obj* o2;
    method* m1;
    method* m2;
    gtk_tree_model_get (model, a, OD_OBJECT, &o1, -1);
    gtk_tree_model_get (model, b, OD_OBJECT, &o2, -1); 
    m1 = o1->method;
    m2 = o2->method;
    if (m1)
	c1 = m1->owner;
    if (m2)
	c2 = m2->owner;    
    cn1 = c1 ? c1->name : Q;
    cn2 = c2 ? c2->name : Q;
    i = strcmp (cn1, cn2);
    if (i)
	return i;
    cn1 = m1 ? m1->jmpname : SYS_INIT;
    cn2 = m2 ? m2->jmpname : SYS_INIT;
    i = strcmp (cn1, cn2);
    return i;
}

static gint ((*obj_comprs[])(GtkTreeModel *model,
			     GtkTreeIter *a,
			     GtkTreeIter *b,
			     gpointer user_data)) = { sort_by_class_name,
						      sort_by_arena, 
						      sort_by_is_array,          /* is_array */
						      sort_by_size,              /* size */
						      sort_by_alloc_class_and_method,  /* alloc_class */
						      sort_by_alloc_class_and_method   /* alloc_method */
			     };



static void append_object_dump (obj* o, wret* w) {
    GtkTreeIter   iter;
    gchar *text[6];
    cls* c = NULL;
    cls* cp = get_class (obj_get_class_id (o));

    /* titles = { "Class name", "arena", "is array", "size", "allocating class", "allocating method"};*/    
    method* m = o->method;
    if (m != NULL)
	c = m->owner;
    text[0] = (cp ? cp->name : Q);
    snprintf (sarena, 64, "%d", o->arena_id);
    text[1] = sarena;
    snprintf (is_array, 64, "%d", (o->is_array ? 1 : 0));
    text[2] = is_array;
    snprintf (size, 64, "%d", o->size);
    text[3] = size;
    text[4] = (c ? c->name : Q);
    text[5] = (m ? m->jmpname : SYS_INIT);

    gtk_list_store_append (w->clist, &iter);
    gtk_list_store_set (w->clist, &iter,
			OD_CLASS, text[0],
			OD_ARENA, text[1],
			OD_IS_ARRAY, text[2],
			OD_SIZE, text[3],
			OD_ALLOC_CLASS, text[4],
			OD_ALLOC_METHOD, text[5],
			OD_OBJECT, o,
			-1);
}

static void print_if_method (obj* o, wret* w) {
    if (o->method == (method*)w->data)
	append_object_dump (o, w);
}

static void print_if_class (obj* o, wret* w) {
    if (obj_get_class_id (o) == ((cls*)w->data)->class_id)
	append_object_dump (o, w);
}

static void add_column (GtkWidget* tree, char* label, int pos, 
			gpointer data, void (*cb)(GtkWidget*, gpointer), 
			gint width) {
    GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
    GtkTreeViewColumn *column = 
	gtk_tree_view_column_new_with_attributes (label, renderer,
						  "text", pos, 
						  NULL );
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
    gtk_tree_view_column_set_resizable (column, TRUE);
    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
    gtk_tree_view_column_set_min_width (column, width);
    if (cb != NULL)
	g_signal_connect (G_OBJECT (column), "clicked", G_CALLBACK (cb), data);
}

static void od_column_clicked (GtkWidget *treeviewcolumn, gpointer user_data) {
    GtkTreeView* tree;
    GtkTreeModel* clist;
    int column;
    tree = GTK_TREE_VIEW (user_data);
    clist = gtk_tree_view_get_model (tree);
    for (column = 0; ; column++) {
	if (treeviewcolumn == (GtkWidget*)gtk_tree_view_get_column (tree, column))
	    break;
    }
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (clist), column, GTK_SORT_ASCENDING);
    gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (clist), column, obj_comprs[column], NULL, NULL);
}

wret create_object_dump_window () {
    GtkWidget *obj_dump;
    GtkWidget *scrolledwindow;
    GtkWidget *tree;
    GtkListStore *clist1;
    GtkTreeSelection *select;
    wret ret;

    obj_dump = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (obj_dump), _("Object dump"));
    scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show (scrolledwindow);
    gtk_container_add (GTK_CONTAINER (obj_dump), scrolledwindow);
    
    clist1 = gtk_list_store_new (ODN_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, 
				 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
				 G_TYPE_STRING, G_TYPE_POINTER);
    tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (clist1));
    add_column (tree, _("Class"), OD_CLASS, (gpointer)tree, od_column_clicked, 200);
    add_column (tree, _("Arena"), OD_ARENA, (gpointer)tree, od_column_clicked, 80);
    add_column (tree, _("Is array"), OD_IS_ARRAY, (gpointer)tree, od_column_clicked, 80);
    add_column (tree, _("Size"), OD_SIZE, (gpointer)tree, od_column_clicked, 80);
    add_column (tree, _("Allocating class"), OD_ALLOC_CLASS, (gpointer)tree, od_column_clicked, 200);
    add_column (tree, _("Allocating method"), OD_ALLOC_METHOD, (gpointer)tree, od_column_clicked, 200);
    gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tree), TRUE);
    gtk_container_add (GTK_CONTAINER (scrolledwindow), tree);    
    select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
    gtk_widget_set_usize (obj_dump, 800, 200);
    ret.window = obj_dump;
    ret.clist = clist1;
    return ret;
}

void show_objects_alloced_by_method (hashtab* objects, method* m) {
    wret w = create_object_dump_window ();
    w.data = m;
    w.row = 0;
    hash_for_each_with_arg ((hash_iter_fa)print_if_method, objects, &w);
    gtk_widget_show_all (w.window);
}

void show_objects_alloced_by_class (hashtab* objects, cls* c) {
    wret w = create_object_dump_window ();
    w.data = c;
    w.row = 0;
    hash_for_each_with_arg ((hash_iter_fa)print_if_class, objects, &w);
    gtk_widget_show_all (w.window);
}
